--[[
Copyright 2010, 2011 William Matthews. All rights reserved.

See LICENSE for license terms.
]]

local CB = select(2,...)
local P = CB.Private
local Items = P:NewModule("Items", "AceEvent-3.0")

local ESSENCE_TRANSFORMATIONS = {
	-- {greater, lesser}
	{52719, 52718}, -- celestial
	{34055, 34056}, -- cosmic
	{22446, 22447}, -- planar
	{16203, 16202}, -- eternal
	{11175, 11174}, -- nether
	{11135, 11134}, -- mystic
	{11082, 10998}, -- astral
	{10939, 10938}, -- magic
}

local ELEMENTAL_TRANSFORMATIONS = {
	-- {big, little, [reversible]}
	{35623, 37700, true}, -- eternal air
	{35624, 37701, true}, -- eternal earth
	{36860, 37702, true}, -- eternal fire
	{35625, 37704, true}, -- eternal life
	{35627, 37703, true}, -- eternal shadow
	{35622, 37705, true}, -- eternal water
	
	{22451, 22572}, -- primal air
	{22452, 22573}, -- primal earth
	{21884, 22574}, -- primal fire
	{21886, 22575}, -- primal life
	{22457, 22576}, -- primal mana
	{22456, 22577}, -- primal shadow
	{21885, 22578}, -- primal water
}

local MISC_TRANSFORMS = {
	{{[34053]=3},{[34052]=1}}, -- small dream shard -> dream shard
	{{[52720]=3},{[52721]=1}}, -- small heavenly shard -> heavenly shard
}

CBD={
	print=function(x) P:DebugPrint(x) end,
	EST=ESSENCE_TRANSFORMATIONS,
	ELT=ELEMENTAL_TRANSFORMATIONS,
}

local function addTransform(from,to)
--	P:DebugPrint({"addTransform",from=from,to=to})
	if not to or not next(to) or next(to,next(to)) then -- for now we only handle one item in "to"
		P:DebugPrint({message="bad size for 'to'",from=from,to=to})
		return
	end
	local fromItem,fromCount = next(from)
	local toItem,toCount = next(to)
	P:GetItemInfo(function(_,fromItemLink)
		P:GetItemInfo(function(_,toItemLink)
			P:DebugPrint("Transform: "..fromItemLink.."x"..fromCount.." -> "..toItemLink.."x"..toCount)
		end,P.genericFailCB,toItem)
	end,P.genericFailCB,fromItem)
	for item in pairs(from) do
		P:GetItemSpell(function(spellname) -- success CB
			local reccb = function(item,count)
--				P:DebugPrint("received item")
				CB:RecordTransaction(nil,from,to)
			end
			local successcb = function(item)
--				P:DebugPrint("cast success")
				P:ReceiveCallback(toItem,toCount,reccb)
			end
			local castcb = function()
				P:SpellCallback(spellname,successcb)
			end
	--		CB:AddItemHandler(i.item,itemcb)
			P:AddSpellHandler(spellname,castcb)			
			P:DebugPrint("Added handler for "..spellname)
		end,function() -- fail CB
			P:DebugPrint("Item has no associated spell: "..item.." not cached?")
		end,item)
	end
end

function Items:OnInitialize()
--	P:Print("Items -- OnInitialize")
	CBD.print("hello world")
	for _, t in pairs(ESSENCE_TRANSFORMATIONS) do
--		P:DebugPrint({essence=t})
		local greater, lesser = {[t[1]]=1}, {[t[2]]=3}
		addTransform(greater,lesser)
		addTransform(lesser,greater)
	end
	for _, t in pairs(ELEMENTAL_TRANSFORMATIONS) do
--		P:DebugPrint({elemental=t})
		local big, little = {[t[1]]=1}, {[t[2]]=10}
		addTransform(little,big)
		if t[3] then
			addTransform(big,little)
		end
	end
	for _, t in pairs(MISC_TRANSFORMS) do
		addTransform(t[1],t[2])
	end
--	P:Print("Items -- OnInitialize Done")
end